import argparse

from counting_lib import explain, generate, predict, test, train


def config():
    parser = argparse.ArgumentParser()
    parser.add_argument("--workdir", type=str, default="./")

    subparsers = parser.add_subparsers(dest="command")
    subparsers.required = True

    subparsers.add_parser("generate")
    subparsers.add_parser("train")
    subparsers.add_parser("explain")

    testing_parser = subparsers.add_parser("test")
    testing_parser.add_argument("--kernel", type=str)
    testing_parser.add_argument("--tau_max", type=int)
    testing_parser.add_argument("--use_model", action="store_true", default=True)

    testing_subparsers = testing_parser.add_subparsers(dest="type")
    testing_subparsers.required = True

    testing_subparsers.add_parser("global")
    testing_subparsers.add_parser("global_cond")
    testing_subparsers.add_parser("local_cond")
    return parser.parse_args()


def main(args):
    workdir = args.workdir

    if args.command == "generate":
        generate(train=True, workdir=workdir)
        generate(train=False, workdir=workdir)
    if args.command == "train":
        train(workdir)
        predict(workdir)
    if args.command == "explain":
        explain(workdir)
    if args.command == "test":
        kernel = args.kernel
        tau_max = args.tau_max

        use_model = args.use_model
        type = args.type
        test(type, workdir, use_model, kernel=kernel, tau_max=tau_max)


if __name__ == "__main__":
    args = config()
    main(args)
